/*************************************************************************
*
*
*    (c) Copyright Olimex 2011
*
*    File name   : main.c
*    Description : Production test for the Olimex LPC-H1343 board
*
*    History :
*    1. Date        : 18 August 2011
*       Author      : Aleksandar Mitev
*       Description : Create
*
**************************************************************************/
#include "includes.h"
#include <yfuns.h>

#define DLY_100US             550

/*variable for clitical section entry control*/
Int32U CriticalSecCntr;
/** external data **/
extern const struct extcon_board extcon_board_def[];

//** internal functions **/
static int MyLowLevelGetchar(void);

/** public functions **/
int can_boardtest(void);

#define LED1_ON() GPIO1DATA &= ~(1 << 10)
#define LED1_OFF() GPIO1DATA |= (1 << 10)

#define LED2_ON() GPIO1DATA &= ~(1 << 11)
#define LED2_OFF() GPIO1DATA |= (1 << 11)

#define GET_B1_BTN() (GPIO0DATA & (1 << 7))
#define GET_WAKEUP_BTN() (GPIO1DATA & (1 << 4))

/*************************************************************************
* Function Name: InitClock
* Parameters: clock, AHB devider
* Return: void
*
* Description: Initialize PLL to desired clock and  AHB devider
*              Sys clock is Sys PLL output
*
*
*************************************************************************/
void InitClock(Int32U clock, Int32U ahbdiv)
{
	/*Sys Oscilator Enable*/
	SYSOSCCLTRL = (MAIN_OSC_FREQ>(20MHZ))?(0x2):(0x0);
	/*Power Up SYS Oscilator*/
	PDRUNCFG_bit.SYSOSC_PD = 0;
	/*Enable Internal RC oscilator*/
	PDRUNCFG_bit.IRC_PD = 0;
	/*Select internal RC oscilator for
	Sys clock source*/
	MAINCLKUEN = 0;
	MAINCLKSEL = 0;
	MAINCLKUEN = 1;
	/*Configure SYS PLL*/
	/*Power Down SYS PLL*/
	PDRUNCFG_bit.SYSPLL_PD = 1;
	/*Select Sys Oscilator for
	SYS PLL source*/
	SYSPLLCLKUEN = 0;
	SYSPLLCLKSEL = 1;
	SYSPLLCLKUEN = 1;
	/*Calc M*/
	Int32U m = clock/MAIN_OSC_FREQ - 1;

	assert(m<32);
	/*Configure PLL frequency*/
	SYSPLLCTRL =  (m)    /*MSEL*/
	|  (0<<5) /*PSEL = 1*/
	|  (0<<7) /*DIRECT = 0*/
	|  (0<<8); /*BYPASS=0*/

	/*Power Up PLL*/
	PDRUNCFG_bit.SYSPLL_PD = 0;
	/*Set Sys AHB Clock devider*/
	SYSAHBCLKDIV_bit.DIV = ahbdiv;
	/*Wain until PLL locks*/
	while(!(SYSPLLSTAT_bit.LOCK));
	/*Select Sys PLL Output for
	Sys clock source*/
	MAINCLKUEN = 0;
	MAINCLKSEL = 3;
	MAINCLKUEN = 1;
}

/*************************************************************************
* Function Name: SYS_GetMainClk
* Parameters: none
* Return: Int32U
*
* Description: return Main Clock [Hz]
*
*************************************************************************/
Int32U SYS_GetMainClk(void)
{
	Int32U Clk;
	switch(MAINCLKSEL_bit.SEL)
	{
	case 0:
		Clk = I_RC_OSC_FREQ;
		break;
	case 1:
		Clk = MAIN_OSC_FREQ;
		break;
	case 2:
		Clk = WDT_OSC_FREQ;
		break;
	case 3:
		switch(SYSPLLCLKSEL_bit.SEL)
		{
		case 0:
			Clk = I_RC_OSC_FREQ;
			break;
		case 1:
			Clk = MAIN_OSC_FREQ;
			break;
		case 2:
			Clk = WDT_OSC_FREQ;
			break;

		default:
			Clk = 0;
			break;
		}

		Clk *= (SYSPLLCTRL_bit.MSEL+1);
		break;

	default:
		Clk = 0;
		break;
	}
	return Clk;
}

void UART_Init (Int32U BaudRate)
{
	Int32U Pclk;
	/*Enable Clocks*/
	UARTCLKDIV = 1;
	/**/
	while((Pclk=SYS_GetMainClk()/UARTCLKDIV)> 70MHZ)
	{
		UARTCLKDIV ++;
	}
	/*Enable UART register clock*/
	SYSAHBCLKCTRL_bit.UART = 1;
	/*Enable I/O Configuration registers*/
	SYSAHBCLKCTRL_bit.IOCON = 1;

	IOCON_PIO1_6 = 1;         //Uart RX function select
	IOCON_PIO1_7 = 1;         //Uart RX function select

	U0LCR_bit.WLS = 3;        //Serlect 8-Bits
	U0LCR_bit.SBS = 0;        //One Stop Bit
	U0LCR_bit.PE = 0;         //No parity

	U0LCR_bit.DLAB = 1;       //Devisor Latch Access ebabled
	//Set UART0 BAUDRATE
	U0DLL = ((((Pclk+8)/16)+BaudRate/2)/BaudRate)%256;  //
	U0DLM = ((((Pclk+8)/16)+BaudRate/2)/BaudRate)/256;  //
	U0LCR_bit.DLAB = 0;           //Clear Devisor Latch Access bit

	U0FCR = (1<<0);     //Enable UART0 FIFOs.
	U0TER = (1<<7);     //Enable Transmission
}

/*************************************************************************
* Function Name: __write
* Parameters: Low Level cahracter output
*
* Return:
*
* Description:
*
*************************************************************************/
size_t __write(int Handle, const unsigned char * Buf, size_t Bufsize)
{
	size_t nChars = 0;

	for (/*Empty */; Bufsize > 0; --Bufsize)
	{
		while( !U0LSR_bit.THRE );  //Wait
		U0THR = * Buf++;
		++nChars;
	}
	return nChars;
}
/*************************************************************************
* Function Name: __read
* Parameters: Low Level cahracter input
*
* Return:
*
* Description:
*
*************************************************************************/
size_t __read(int handle, unsigned char * buffer, size_t size)
{
	int nChars = 0;

	/* This template only reads from "standard in", for all other file
	* handles it returns failure. */
	if (handle != _LLIO_STDIN)
	{
		return _LLIO_ERROR;
	}

	for (/* Empty */; size > 0; --size)
	{
		int c = MyLowLevelGetchar();
		if (c < 0)
		break;

		*buffer++ = c;
		++nChars;
	}

	return nChars;
}

/*************************************************************************
* Function Name: DelayResolution100us
* Parameters: Int32U Dly
* Return: void
*
* Description: Delay Dly * [100us]
*		
*************************************************************************/
void DelayResolution100us(Int32U Dly)
{
	for(; Dly; Dly--)
	{
		for(volatile Int32U j = DLY_100US; j; j--)
		{
		}
	}
}

/*************************************************************************
* Function Name: GpioInit
* Parameters: void
* Return: void
*
* Description: Reset all GPIO pins to default: primary function
*
*************************************************************************/
void GpioInit(void)
{
	// Set to inputs
	GPIO0DIR = 0;
	GPIO1DIR = 0;
	GPIO2DIR = 0;
	GPIO3DIR = 0;

	// Reset all GPIO pins to default primary function
	IOCON_PIO2_6 = 0xC0;
	IOCON_PIO2_0 = 0xC0;
	IOCON_RESET_PIO0_0 = 0xC0;
	IOCON_PIO0_1 = 0xC0;
	IOCON_PIO1_8 = 0xC0;
	IOCON_PIO0_2 = 0xC0;
	IOCON_PIO2_7 = 0xC0;
	IOCON_PIO2_8 = 0xC0;
	IOCON_PIO2_1 = 0xC0;
	IOCON_PIO0_3 = 0xC0;
	IOCON_PIO1_9 = 0xC0;
	IOCON_PIO2_4 = 0xC0;
	IOCON_PIO2_5 = 0xC0;
	IOCON_PIO0_6 = 0xC0;
	IOCON_PIO0_7 = 0xC0; // B1
	IOCON_PIO2_9 = 0xC0;
	IOCON_PIO2_10 = 0xC0;
	IOCON_PIO2_2 = 0xC0;
	IOCON_PIO0_8 = 0xC0;
	IOCON_SWCLK_PIO0_10 = 0xC0;
	IOCON_PIO2_11 = 0xC0;
	IOCON_PIO3_0 = 0xC0;
	IOCON_PIO3_1 = 0xC0;
	IOCON_PIO2_3 = 0xC0;
	IOCON_PIO3_2 = 0xC0;
	IOCON_PIO1_5 = 0xC0;
	IOCON_PIO3_3 = 0xC0;
	IOCON_PIO0_9 = 0xC0;

	IOCON_PIO1_6 = 0xC0;
	IOCON_PIO1_7 = 0xC0;

	
	IOCON_PIO1_10 = 0xC0; // LED2
	IOCON_R_PIO0_11 = 0xC0;
	IOCON_R_PIO1_0 = 0xC0;
	IOCON_R_PIO1_1 = 0xC0;
	IOCON_R_PIO1_2 = 0xC0;
	IOCON_SWDIO_PIO1_3 = 0xC0;
	IOCON_PIO1_4 = 0xC0; // WAKE_UP button
	IOCON_PIO1_11 = 0xC0; // LED1

	IOCON_PIO0_4 = 0x00;
	IOCON_PIO0_5 = 0x00;
	
	// co0nfigure LEDS
	GPIO1DIR |= (1 << 10) | (1 << 11);
	LED1_OFF();
	LED2_OFF();
	
}

/*************************************************************************
* Function Name: main
* Parameters: none
*
* Return: none
*
* Description: main
*
*************************************************************************/
int main(void)
{
	/*Init clock*/
	InitClock(132MHZ, 2);
	/*Enable GPIO Clock*/
	SYSAHBCLKCTRL_bit.GPIO = 1;
	/*Enable IOCON Clock*/
	SYSAHBCLKCTRL_bit.IOCON = 1;
	/*Set all pins as input ports*/
	GpioInit();
	/*UART Init: 115200 bps, 8-bit,
	1-Stop bit, Parity - None*/
	UART_Init(115200);
	
	// only run production test if jumper for PIO0_3 is closed!
	if( !(GPIO0DATA & (1 << 3)) ) {
	
		printf("\n\r");
		printf("\n\r");
		printf("***************************\n\r");
		printf("*     OLIMEX LPC-P11C14   *\n\r");
		printf("* Production Test         *\n\r");
		printf("***************************\n\r");
		
		/* TEST EXTENSION */
		extcon_boardtest(extcon_board_def);
	
		/* TEST CAN */	
		printf("\n\r");
		printf("\n\r");
		can_boardtest();
	}

	GpioInit();
	/*UART Init: 115200 bps, 8-bit,
	1-Stop bit, Parity - None*/
	UART_Init(115200);
	
	/* TEST BUTTONS AND LEDS */
	printf("Check buttons and LEDs!");
	/*Set all pins as input ports*/

	
	int blink = 0;
	while(1) {
		DelayResolution100us(5);
		blink++;
		
		if( GET_B1_BTN() ) {
			LED1_ON();
		} else {
			if(blink & (1 << 10)) {
				LED1_ON();
			} else {
				LED1_OFF();
			}
		}
		
		if( GET_WAKEUP_BTN() ) {
			LED2_ON();
		} else {
			if(blink & (1 << 10)) {
				LED2_OFF();
			} else {
				LED2_ON();
			}
		}
	}
}

int can_boardtest(void)
{
	// we shall use the built in test capabilities of the CAN module
	// in TEST mode there is the CANTEST register where we can set the
	// state of the TX pin and monitor the state of the RX pin

	int error;
	int i;	
	
	// start clocks
	PRESETCTRL |= (0x1<<3);
	SYSAHBCLKCTRL |= (1<<17);

	// enter test mode
	CANCNTL_bit.TEST = 1;
	
	do {
		error = 0;

		printf("\n\rStarting CAN pins test...");
		
		printf("\n\rMove jumper position to SILENT and press space...");
		while(getchar() != ' ');
	
		for(i = 0; i < 5; i++) {
			int state;
			
			CANTEST_bit.TX = 0x2; // LOW
			DelayResolution100us(5);
			
			state = CANTEST_bit.RX;
			
			CANTEST_bit.TX = 0x3; // HI
			DelayResolution100us(5);
	
			// RX should be static
			if(CANTEST_bit.RX != state)
				error |= -1;
		}

		printf("\n\rMove jumper position to NORMAL and press space...");
		while(getchar() != ' ');
	
		for(i = 0; i < 5; i++) {
			CANTEST_bit.TX = 0x2; // LOW
			DelayResolution100us(5);
			
			// RX should also be LOW
			if(CANTEST_bit.RX)
				error |= -1;
			
			CANTEST_bit.TX = 0x3; // HI
			DelayResolution100us(5);
	
			// RX should also be HI
			if(!CANTEST_bit.RX)
				error |= -1;
		}
			
		printf("\n\rShort CANL and CANH and press space...");
		while(getchar() != ' ');
	
		for(i = 0; i < 5; i++) {
			int state;
			
			CANTEST_bit.TX = 0x2; // LOW
			DelayResolution100us(5);
			
			state = CANTEST_bit.RX;
			
			CANTEST_bit.TX = 0x3; // HI
			DelayResolution100us(5);
	
			// RX should be static
			if(CANTEST_bit.RX != state)
				error |= -1;
		}
		
		if(error == 0) {
			printf("\n\rCAN test successful :)");
			printf("\n\r");
			break;
		}
		
		printf("\n\rERROR in test!! Press space to run again or 'n' to continue...");
		
		{
			char input;
			while( (input = getchar()) == 0xFF );
			if( (input == 'n') || (input == 'N') ) {
				break;
			}
		}
	} while(1);
	
	return error;
	
}


/** private functions **/
static int MyLowLevelGetchar(void)
{
int ch;

  if(U0LSR_bit.DR)
  {
    ch = U0RBR;
    if(U0LSR_bit.OE || U0LSR_bit.PE || U0LSR_bit.FE )
    {
      return (ch | 0x10000000);
    }
    return (ch & 0xff );
  }
  return -1;
}

