/****************************************************************************
 *
 * Project: Generic test routines.
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: extcon.h
 * Description: External connector test configuration for LPC-P2919
 * Developer: Dimitar Dimitrov ( dinuxbg,gmail.com )
 *
 * Last change: $Date: 2010/10/26 $
 * Revision: $Revision: #1 $
 * Id: $Id: //depot/IAR/auxiliary_arm2_5_xx/TestCode/NXP/LPC12xx/IAR-LPC-122x-SK/modules/extcon-conf.c#1 $
 *
 ****************************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <assert.h>
#include <nxp/iolpc1343.h>

#include "extcon.h"


#define ARRAY_NUMELEM(A)	(sizeof((A))/sizeof((A)[0]))


#define PIN(CON_PIN, PRT, PRT_PIN, CHIP_PIN)	\
		{ \
			.con_pin = CON_PIN, \
			.port_pin = ((PRT)<<8) | (PRT_PIN), \
			.chip_pin = CHIP_PIN, \
		}

static void print_port_pin(char *buf, const struct extcon_pin *pin)
{
	sprintf(buf, "P%d[%d]", pin->port_pin>>8, pin->port_pin & 0xff);
}

static int lpc13xx_gpio_read(const struct extcon_pin *pin)
{
  volatile unsigned long * port = (volatile unsigned long * )\
                                  (0x50003FFC + (pin->port_pin>>8)*0x10000);

  const uint32_t pin_mask = 1 << (pin->port_pin & 0xff);

	if(*port & pin_mask) return (1);
  else             return (0);
}


static void lpc13xx_gpio_set_inp(const struct extcon_pin *pin)
{
	const uint32_t pin_mask = 1 << (pin->port_pin & 0xff);
  volatile unsigned long * gpiodir = (volatile unsigned long * )\
                                  (0x50008000 + (pin->port_pin>>8)*0x10000);

	*gpiodir &= ~pin_mask;
}


static void lpc13xx_gpio_set_outp(const struct extcon_pin *pin, int val)
{
	const uint32_t pin_mask = 1 << (pin->port_pin & 0xff);
  volatile unsigned long * gpiodir = (volatile unsigned long * )\
                                  (0x50008000 + (pin->port_pin>>8)*0x10000);
  volatile unsigned long * port = (volatile unsigned long * )\
                                  (0x50003FFC + (pin->port_pin>>8)*0x10000);

	*gpiodir |= pin_mask;

	if(val) {
		*port |= pin_mask;
	} else {
		*port &= ~pin_mask;
	}
}

  	                                               /*CONN_PIN   PORT_PIN  CHIP_PIN*/
static const struct extcon_pin extcon_main_outp = PIN(	42,	1,	11,	42);

static const struct extcon_pin extcon_pins[] = {
	/* CONN_PIN   PORT_PIN  CHIP_PIN */
	PIN(	36,	3,	0,	36),
	PIN(	37,	3,	1,	37),
	PIN(	43,	3,	2,	43),
	PIN(	48,	3,	3,	48),

	PIN(	2,	2,	0,	2),
	PIN(	13,	2,	1,	13),
	PIN(	26,	2,	2,	26),
	PIN(	38,	2,	3,	38),
	PIN(	1,	2,	6,	1),
	PIN(	11,	2,	7,	11),
	PIN(	12,	2,	8,	12),
	PIN(	25,	2,	10,	25),
	PIN(	31,	2,	11,	31),

//	PIN(	3,	0,	0,	3), -> RESET
	PIN(	4,	0,	1,	4),
	PIN(	10,	0,	2,	10),
	PIN(	14,	0,	3,	14),
	PIN(	15,	0,	4,	15),
	PIN(	16,	0,	5,	16),
	PIN(	23,	0,	6,	23),
	PIN(	24,	0,	7,	24),
	PIN(	27,	0,	8,	27),
	PIN(	28,	0,	9,	28),
//	PIN(	29,	0,	10,	29), -> SWCLK
	PIN(	32,	0,	11,	32),

	PIN(	33,	1,	0,	33),
	PIN(	34,	1,	1,	34),
	PIN(	35,	1,	2,	35),
//	PIN(	39,	1,	3,	39), -> SWDIO
	PIN(	40,	1,	4,	40),
	PIN(	45,	1,	5,	45),
//	PIN(	46,	1,	6,	46), -> RXD
//	PIN(	47,	1,	7,	47), -> TXD
	PIN(	9,	1,	8,	9),
//	PIN(	30,	1,	10,	30), -> LED2
};

  	                                               /*CONN_PIN   PORT_PIN  CHIP_PIN*/
static const struct extcon_pin uext_main_outp = PIN(	9,	2,	1,	13);

static const struct extcon_pin uext_pins[] = {
	/* CONN_PIN   PORT_PIN  CHIP_PIN */
//	PIN(	3,	1,	7,	47), -> TXD
//	PIN(	4,	1,	6,	46), -> RXD
	PIN(	5,	0,	4,	15),
	PIN(	6,	0,	5,	16),
	PIN(	7,	2,	2,	26),
	PIN(	8,	2,	3,	38),
	PIN(	10,	2,	0,	2),
};

static const struct extcon_jack extcons[] = {
	{
		.name = "EXT",
		.main_outp_pin = &extcon_main_outp,
		.pins = extcon_pins,
		.num_pins = ARRAY_NUMELEM(extcon_pins)
	},
	{
		.name = "UEXT",
		.main_outp_pin = &uext_main_outp,
		.pins = uext_pins,
		.num_pins = ARRAY_NUMELEM(uext_pins)
	},
};

static void setup_delay(void)
{
	for(volatile int i=0; i<50000; i++);
}

const struct extcon_board extcon_board_def = {
	.name = "LPC-P11C14",
	.connectors = extcons,
	.num_connectors = ARRAY_NUMELEM(extcons),

	.gpio_read = lpc13xx_gpio_read,
	.gpio_set_inp = lpc13xx_gpio_set_inp,
	.gpio_set_outp = lpc13xx_gpio_set_outp,

	.print_port_pin = print_port_pin,
	.print_con_pin = 0,	/* use default implementation */
	.print_chip_pin = 0,	/* use default implementation */
	
	.setup_delay = setup_delay,
	
	.puts = 0,
};



